#!/usr/bin/perl -w
use PDA::Pilot; # from the pilot-link package.
use POSIX qw(mktime tzset);
use Locale::Recode;

our %venues;
our %teams;
our %clubs;

die "required argument missing\n" if ($#ARGV < 0);
$abbr = $ARGV[0];

$dir = `pwd`;
chomp($dir);
require "$dir/fixture.$abbr.txt";

$ENV{TZ} = 'UTC';
POSIX::tzset();

my $cd = Locale::Recode->new (to   => 'CP1252',
                              from => 'UTF-8');
die $cd->getError if $cd->getError;
                              

# venue_map: international name => 'l' localized name, 'c' capacity, 'n'
# internal number.
               
%venue_map = (
  'Basel' =>      { 'c' => 42500, 'x' =>  26, 'y' => 30, 'n' => 1 },
  'Geneva' =>     { 'c' => 30000, 'x' =>   7, 'y' => 55, 'n' => 2 },
  'Vienna' =>     { 'c' => 50000, 'x' => 136, 'y' => 15, 'n' => 3 },
  'Klagenfurt' => { 'c' => 30000, 'x' => 113, 'y' => 45, 'n' => 4 },
  'Zrich' =>     { 'c' => 30000, 'x' =>  39, 'y' => 34, 'n' => 5 },
  'Berne' =>      { 'c' => 32000, 'x' =>  24, 'y' => 41, 'n' => 6 },
  'Innsbruck' =>  { 'c' => 30000, 'x' =>  75, 'y' => 36, 'n' => 7 },
  'Salzburg' =>   { 'c' => 30000, 'x' =>  96, 'y' => 25, 'n' => 8 },
);
$venue_map{$_}->{l} = $venues{$_} for (keys(%venue_map));

# team_map: abbreviation => 'l' localized name, 'c' comment, 'n' internal
# number; the number plays an important role for the last tie breaker "UEFA
# coefficient"
%team_map = (
  'CZE' => { 'c' => '', 'n' => 0 },
  'POR' => { 'c' => '', 'n' => 1 },
  'TUR' => { 'c' => '', 'n' => 2 },
  'SUI' => { 'c' => '', 'n' => 3 },

  'CRO' => { 'c' => '', 'n' => 4 },
  'GER' => { 'c' => '', 'n' => 5 },
  'POL' => { 'c' => '', 'n' => 6 },
  'AUT' => { 'c' => '', 'n' => 7 },

  'NED' => { 'c' => '', 'n' => 8 },
  'ITA' => { 'c' => '', 'n' => 9 },
  'FRA' => { 'c' => '', 'n' => 10 },
  'ROU' => { 'c' => '', 'n' => 11 },

  'SWE' => { 'c' => '', 'n' => 12 },
  'ESP' => { 'c' => '', 'n' => 13 },
  'GRE' => { 'c' => '', 'n' => 14 },
  'RUS' => { 'c' => '', 'n' => 15 },

  'YUG' => { 'c' => '', 'n' =>17 },
  'USS' => { 'c' => '', 'n' =>18 },
  'HUN' => { 'c' => '', 'n' =>19 },
  'DEN' => { 'c' => '', 'n' =>20 },
  'ENG' => { 'c' => '', 'n' =>21 },
  'BEL' => { 'c' => '', 'n' =>22 },
  'IRL' => { 'c' => '', 'n' =>23 },
  'SCO' => { 'c' => '', 'n' =>24 },
  'GUS' => { 'c' => '', 'n' =>25 },
  'BUL' => { 'c' => '', 'n' =>26 },
  'NOR' => { 'c' => '', 'n' =>27 },
  'SLO' => { 'c' => '', 'n' =>28 },
  'LAT' => { 'c' => '', 'n' =>29 }, 
);
$team_map{$_}->{l} = $teams{$_} for (keys(%team_map));

sub UInt32
{
  return sprintf("%c%c%c%c", ($_[0] & 0xff000000)>>24,
		 ($_[0] & 0xff0000)>>16,
		 ($_[0] & 0xff00)>>8,
		 ($_[0] & 0xff));
}

sub UInt16
{
  return sprintf("%c%c", ($_[0] & 0xff00)>>8,
		 ($_[0] & 0xff));
}

sub UInt8
{
  return sprintf("%c", $_[0]);
}

$secs_since_70 = 24107 * 86400;

%info = ("type"=>"Data","creator"=>"tpE8","name"=>"euro2008DB");
$pdb = PDA::Pilot::File::create("euro2008DB.pdb", \%info )
  or die "something went wrong\n";

$rec = "\1";
$pdb->addRecordRaw($rec,0,0,0);
$rec = "";

# player position/function
%posEnum = ('G'=>0,'D'=>1,'M'=>2,'F'=>3,'C'=>4);

open FILE,"matches.txt" or die "oops\n";
while (<FILE>) {
    chomp();
    @args = split(/:/);
    $date = "";
    if ($args[6] && $args[6] =~ /^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})/) {
        $date = mktime(0, $5, $4, $3, $2-1, $1-1900) + $secs_since_70;
    }
    $args[6] = UInt32($date);
    $args[5] = $venue_map{$args[5]}->{n};
    $args[3] = 255 if ($args[3] == -1);
    $args[4] = 255 if ($args[4] == -1);
    $args[1] = $team_map{$args[1]}->{n}%4+1 if ($args[0]>0);
    $args[2] = $team_map{$args[2]}->{n}%4+1 if ($args[0]>0);
    $rec .= sprintf("%c%c%c%c%c%c%s", @args);
}
close FILE;
$pdb->addRecordRaw($rec,1,0,0);
$rec = '';

for (sort { $venue_map{$a}->{n} <=> $venue_map{$b}->{n} } keys %venue_map) {
    $rec .= $venue_map{$_}->{l} 
         .  "\0" x (30-length($venue_map{$_}->{l})) 
         .  UInt32($venue_map{$_}->{c})
         .  UInt8($venue_map{$_}->{x}) . UInt8($venue_map{$_}->{y});
}
$pdb->addRecordRaw($rec,2,0,0);
$rec = '';

for (sort { $team_map{$a}->{n} <=> $team_map{$b}->{n} } keys %team_map) {
    $rec .= $team_map{$_}->{l} . "\0" x (20-length($team_map{$_}->{l})) . 
            $_                 . "\0" x (4-length($_)) .
            $team_map{$_}->{c} . "\0" x (80-length($team_map{$_}->{c}));
}
$pdb->addRecordRaw($rec,3,0,0);
$rec = '';

$recno = 4;
open FILE, "teams.txt" or die "File teams.txt is missing.\n";
while (<FILE>) {
  chomp;
  if (/^$/) {
      $pdb->addRecordRaw($rec,$recno,0,0);
      $recno++;
      $rec = "";
  }
  @args = split(/:/);
  next if ($#args<1);
  $date = 0;
  if ($args[4] && $args[4] =~ /^(\d{4})(\d{2})(\d{2})/) {
    $date = $3 + ($2<<5) + (($1-1904)<<9);
  }

  if (length($args[1])>29 or length($args[2])>27 or length($args[3])!=3) {
    print "too long: $args[1] (".length($args[1]).")/$args[2] (".length($args[2]).")\n";
    next;
  }
  $args[2] = $clubs{$args[2]} if (defined($clubs{$args[2]}));
  $cd->recode($args[1]);
  $cd->recode($args[2]);
  
  $args[5] = 0 unless defined($args[5]);
  $rec .= $args[1] . "\0" x (30-length($args[1])) .
          $args[2] . "\0" x (28-length($args[2])) .
          $args[3] . "\0" x  (4-length($args[3])) .
          UInt16($date) .
          chr($posEnum{$args[0]}) . chr($args[5]);
}
