/*
 * Euro 2008 is Copyright (c) 2008 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "euro2008.h"
#include "euro2008rcp.h"

DmOpenRef euro2008DB, euro2008Pics, datebook;
struct prefs thePrefs;

static LocalID theId;
static UInt16  theCard;

Boolean OpenDatebook()
{
  LocalID id = 0;
  DmSearchStateType state; 
  UInt16 card;
  Err theErr;

  /*
   * Try to open the datebook database
   */
  datebook = 0;
  theErr = DmGetNextDatabaseByTypeCreator(true, &state, 'DATA', 
					  'date', true, &card, &id); 
  if ( (theErr != dmErrCantFind) ) {
    datebook = DmOpenDatabase(card, id, dmModeReadWrite);
    return true;
  }
  return false;
}

Boolean OpenDatabase()
{
  LocalID id = 0;
  DmSearchStateType state; 
  UInt16 card;
  Err theErr;
  MemHandle handle;
  char *p;

  /*
   * Open the main database; it includes the schedule, team names etc.
   */
  theErr = DmGetNextDatabaseByTypeCreator(true, &state, 'Data', 
					  'tpE8', true, &card, &id); 

  if ( (theErr == dmErrCantFind) || 
       (0 == (euro2008DB = DmOpenDatabase(card, id, dmModeReadWrite)) ) ) {
    DisplayErrorMessage(IdErrorMsg);
    return false;
  }

  if (! (handle = DmQueryRecord(euro2008DB, 0)) ) {
    DisplayErrorMessage(IdErrorMsg+1);
    DmCloseDatabase(euro2008DB);
    return false;
  }

  theId = id;
  theCard = card;

  p = MemHandleLock(handle);
  MemMove(&thePrefs, p, sizeof(thePrefs));
  MemHandleUnlock(handle);
  
  if (thePrefs.version != 1) {
    DisplayErrorMessage(IdErrorMsg+2);
    DmCloseDatabase(euro2008DB);
    return false;
  }

  /*
   * Try to open the optional Pictures database
   */
  theErr = DmGetNextDatabaseByTypeCreator(true, &state, 'Pics', 
					  'tpE8', true, &card, &id); 
  if ( (theErr != dmErrCantFind) )
    euro2008Pics = DmOpenDatabase(card, id, dmModeReadOnly);
  
  return true;
}

char* ReadFixture(char* pos, struct matchRecord* matchRec)
{
  MemHandle handle;
  static char *start;
  static UInt32 size;
  if (!pos) {
    handle = DmQueryRecord(euro2008DB,1);
    start = pos = MemHandleLock(handle);
    size = MemHandleSize(handle);
  } else {
    pos += sizeof(struct matchRecord);
  }
  if (pos-start>=size) {
    MemPtrUnlock(start);
    return 0;
  }
  MemMove(matchRec, pos, sizeof(struct matchRecord));
  return pos;
}

void GetMatch(UInt16 num, struct matchRecord* matchRec)
{
  char *p = MemHandleLock(DmQueryRecord(euro2008DB,1));
  MemMove(matchRec, 
	  p+sizeof(struct matchRecord)*num, 
	  sizeof(struct matchRecord));
  MemPtrUnlock(p);
  return;
}

void WriteFixture(struct match *match, UInt16 num, UInt16 group)
{
  struct matchRecord matchRec;
  int i;
  char *p = MemHandleLock(DmGetRecord(euro2008DB,1));

  for (i=0; i<num; i++) {
    matchRec.group = group;
    matchRec.date = match[i].date;
    matchRec.venue = match[i].venue;
    matchRec.e1 = match[i].erg1;
    matchRec.e2 = match[i].erg2;
    matchRec.m1 = match[i].team1;
    matchRec.m2 = match[i].team2;
    DmWrite(p, 
	    match[i].recNo*sizeof(struct matchRecord), 
	    &matchRec, 
	    sizeof(struct matchRecord));
  }
  MemPtrUnlock(p);
  DmReleaseRecord(euro2008DB,1,true);
}

void SetHandleString(FieldType *fptr, int num)
{
  MemHandle handle;
  char *p;
  struct team *team;

  if ( (handle = DmQueryRecord(euro2008DB,3)) ) {
    p = MemHandleLock(handle);
    MemHandleUnlock(handle);
    p += num*sizeof(struct team);
    team = (struct team*) p;
    SetTextFieldWithString(fptr, team->name, false);
  }

}

struct team* GetTeamString(int num)
{
  MemHandle handle;
  char *p;
  struct team *team;

  if ( num && (handle = DmQueryRecord(euro2008DB,3)) ) {
    p = MemHandleLock(handle);
    p += (num-1)*sizeof(struct team);
    team = (struct team*) p;
    MemHandleUnlock(handle);
    return team;
  }
  return 0;
}

UInt16 ComputeSetHandleString(FieldType *fptr, int num)
{
  char *p, buffer[20];
  struct match match[6];
  struct teamState table[4];
  struct matchRecord matchRec;
  int platz, group, final, rec, newnum;

  if (num>10 && num<25) {
    platz = num / 10;
    group = num % 10;
    ReadSetFixture(match,group,false);
    final = ComputeTable(table,match,0xffffffff);
    if (!final) {
      if (fptr) {
	p = GetString(IdDefaultName);
	StrIToA(buffer,platz);
	StrCat(buffer,". ");
	StrCat(buffer,p);
	StrCat(buffer,"  ");
	buffer[StrLen(buffer)-1] = 'A'-1+group;
	SetTextFieldWithString(fptr,buffer,false);
      }
      // indicates that the finalalists are yet unknown
      return num+64;
    }

    if (fptr)
      SetHandleString(fptr, (group-1)*4+table[platz-1].teamNo-1);
    return (group-1)*4+table[platz-1].teamNo;
  } 

  if (num>=25) {
    platz = 1;
    rec   = num - 1;
    GetMatch(rec,&matchRec);
    newnum = num;
    if (platz == 1) {
      if (matchRec.e1 > matchRec.e2) {
	newnum = matchRec.m1;
      }
      if (matchRec.e1 < matchRec.e2) {
	newnum = matchRec.m2;
      }
    }
    if (platz == 2) {
      if (matchRec.e1 > matchRec.e2) {
	newnum = matchRec.m2;
      }
      if (matchRec.e1 < matchRec.e2) {
	newnum = matchRec.m1;
      }
    }
    if (newnum<num) 
      return ComputeSetHandleString(fptr,newnum);
    else {
      if (fptr) {
	p = GetString(IdDefaultName+1);
	StrIToA(buffer,platz);
	StrCat(buffer,". ");
	StrCat(buffer,p);
	StrCat(buffer," ");
	StrIToA(buffer+StrLen(buffer),rec+1);
	SetTextFieldWithString(fptr,buffer,false);
      }
    }
  }
  return rec+64;
}

UInt32 SetLocationString(Int16 id, int num)
{
  MemHandle handle;
  char *p;
  struct venue *venue;
  FieldType *fldP = Id2Ptr(id);

  if ( (handle = DmQueryRecord(euro2008DB,2)) ) {
    p = MemHandleLock(handle);
    p += num*sizeof(struct venue);
    venue = (struct venue*) p;
    FldSetTextPtr(fldP, venue->name);
    MemHandleUnlock(handle);
    FldDrawField(fldP);
    return venue->cap;
  }
  return 0;
}

struct venue* GetLocation(int num)
{
  MemHandle handle;
  char *p;
  struct venue *venue;

  if ( (handle = DmQueryRecord(euro2008DB,2)) ) {
    p = MemHandleLock(handle);
    p += num*sizeof(struct venue);
    venue = (struct venue*) p;
    MemHandleUnlock(handle);
    return venue;
  }
  return 0;
}

UInt16 readPlayers(char *items[], UInt16 selectedTeam)
{
  UInt16 i = 0;
  MemHandle handle;
  char *p, *end;

  if ( (handle = DmQueryRecord(euro2008DB,3+selectedTeam)) ) {
    p = MemHandleLock(handle);
    end = p+MemHandleSize(handle);
    while (p<end) {
      items[i++] = p;
      p += sizeof(struct player);
    }
    MemHandleUnlock(handle);
  }
  return i;
}

void SaveStatus(void)
{
  UInt16 dbAttr;
  if (errNone == DmDatabaseInfo(theCard,theId,NULL,&dbAttr,
				NULL,NULL,NULL,NULL,NULL,NULL,
				NULL,NULL,NULL)) {
    dbAttr |= dmHdrAttrBackup;
    DmSetDatabaseInfo (theCard,theId,NULL,&dbAttr,
		       NULL, NULL, NULL, NULL, NULL, NULL, 
		       NULL, NULL, NULL);
  }
  // close all previously opened databases
  if (euro2008DB)
    DmCloseDatabase(euro2008DB);
  if (euro2008Pics)
    DmCloseDatabase(euro2008Pics);
  PrefSetAppPreferences('tpE8', 0, appPrefs.version, 
			&appPrefs, sizeof(appPrefs), false);
}

/*
 * Beaming support
 */
static Err WriteDBData(const void *dataP, UInt32 *sizeP, void *userDataP)
{
  Err err;
  *sizeP = ExgSend((ExgSocketType*)userDataP,(void*)dataP,*sizeP,&err);
  return err;
}

void BeamDatabase(void)
{
  ExgSocketType s;
  char name[] = "euro2008DB.pdb";
  Err err;

  if ( !FeatureSet30 ) {
    DisplayInformationMessage(IdErrorMsg+5);
    return;
  }

  MemSet(&s, sizeof(s), 0);
  s.description = name;
  s.name = name;

  if (! (err = ExgPut(&s)) ) {
    err = ExgDBWrite(WriteDBData,&s,NULL,theId,theCard);
  }
  ExgDisconnect(&s, err);
}
