#ifndef _EURO2008_H_
#define _EURO2008_H_

/*
 * constants for querying the 5-way navigator.
 */
#define fiveWayLeft   1028
#define fiveWayRight  2056
#define fiveWaySelect 4096

/*
 * main.c
 */
extern UInt32 romversion; 

#define FeatureSet20 (romversion >= 0x02003000)
#define FeatureSet30 (romversion >= 0x03003000)
#define FeatureSet31 (romversion >= 0x03103000)
#define FeatureSet35 (romversion >= 0x03503000)

/*
 * utils.c
 */
typedef void FormInitFunctionType(void);
char* GetString(UInt16 id);
void DisplayErrorMessage(UInt16 id);
void DisplayInformationMessage(UInt16 id);
UInt16 DisplayConfirmationMessage(UInt16 id);
void* Id2Ptr(UInt16 id);
void FrmMoveObject(Int16 Id,Int16 xd,Int16 yd);
void SetHandleInt(FieldType* fptr, Int32 n);
void SetTextFieldWithHandle(FieldPtr textFldP, MemHandle textH, Boolean drawIt);
Err SetTextFieldWithString(FieldPtr textFldP, char *string, Boolean drawIt);
void DrawButton(Int16 res, Int16 bitmapid);
void FldRecalculateScrollbar(FieldPtr fld, ScrollBarPtr scl);
void FldDoScrolling(FieldPtr fld, ScrollBarPtr scl);
void MyDoDialog(UInt16, FormEventHandlerType*, FormInitFunctionType*);

/*
 * euro2008.c
 */
extern Char* progDatePtr;

typedef struct {
  Int16 version;
  Int16 options;
#define OPT_UseFrmMatches          1
#define OPT_AlternativeGroupLabels 2
  Int32 gmtOff;
  Int16 group, round;
} MyAppPreferencesType;

struct player {
  char name[30];
  char team[28];
  char country[4];
  DateType dateBirth;
  enum { Goal, Defense, Midfield, Forward, Coach } position;
  Int8 number;
};

struct team {
  char name[20];
  char abbr[4];
  char info[80];
};

struct venue {
  char name[30];
  UInt32 cap;
  UInt8 x,y;
};

struct teamState {
  UInt16 Goals, GoalsAgainst, Points;
  Int8 Matches, Position, Won, Draw, Lost;
  UInt8 teamNo;
};

struct matchRecord {
  UInt8 group, m1, m2;
  Int8 e1, e2;
  UInt8 venue;
  UInt32 date;
};

struct match {
  FieldType *m1, *m2, *e1, *e2, *num;
  UInt8 team1, team2, team1Final, team2Final;
  Int8 erg1, erg2, venue;
  UInt32 date;
  UInt8 recNo;
};

struct history {
  UInt16 date;  // days since 1904-1-1
  UInt8 venue,  // number of event; is consistent with the position of the year
                // in the IdFrmMain+3 list, i.e. 1960 is number 12.
        phase,  // 1-4: group number; 16-19: quarter, semi, place, final
        m1, m2, // number of team
        e1,     // result; if &64, overtime; if &128, penalty shooting.
        e2;
};

Int16 ComputeTable(struct teamState *table, struct match *match, UInt32 mask);
void ReadSetFixture(struct match* match, UInt16 group, Boolean setallfields);
Boolean TimezoneFormHandleEvent(EventPtr e);
Boolean AboutFormHandleEvent(EventPtr e);
Boolean MatchdayFormHandleEvent(EventPtr e);
Boolean MainPopupFormHandleEvent(EventPtr e);

extern UInt16 selectedTeam;
extern Int16 selectedMatch;
extern Int16 selectedGroup;
extern Int32 defaultDateFormat;
extern MyAppPreferencesType appPrefs;
extern struct match match[8];

/*
 * database.c
 */
struct prefs {
  UInt8 version;
};
extern struct prefs thePrefs;
extern DmOpenRef euro2008DB, euro2008Pics, datebook;

Boolean OpenDatebook(void);
Boolean OpenDatabase(void);
char* ReadFixture(char*, struct matchRecord*);
void WriteFixture(struct match *match, UInt16 num, UInt16 group);
void SetHandleString(FieldType *fptr, int num);
UInt16 ComputeSetHandleString(FieldType *fptr, int num);
struct team* GetTeamString(int num);
UInt32 SetLocationString(Int16 id, int num);
struct venue* GetLocation(int num);
UInt16 readPlayers(char *items[], UInt16 selectedTeam);
void SaveStatus(void);
void BeamDatabase(void);

/*
 * teams.c
 */
Boolean TeamsFormHandleEvent(EventPtr e);
Boolean PlayerFormHandleEvent(EventPtr e);

/*
 * stadium.c
 */
Boolean ShowStadiumFormHandleEvent(EventPtr e);

/*
 * matches.c
 */
void SetMatchesListTeamString(Int16 num, char *p, char *buf);
void MakeDatebookEntry(UInt32 secs, Int16 n);
Boolean MatchesFormHandleEvent(EventPtr e);

/*
 * fixture.c
 */
Boolean FixtureFormHandleEvent(EventPtr e);

#endif
