/*
 * Euro 2008 is Copyright (c) 2008 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "euro2008.h"
#include "euro2008rcp.h"

/**
 * Show Fixture Overview Form
 */

static void DrawFixtureTableItem(void* tableP, Int16 row, Int16 column, 
				 RectangleType *bounds)
{
  Char buf[4];
  Int16 n = TblGetItemInt(tableP, row, column);
  struct team *team = GetTeamString(n);
  MemMove(buf,team->abbr,4);
  if (row<2) {
    FntSetFont(boldFont);
  }
  WinDrawChars(buf, StrLen(buf), bounds->topLeft.x, bounds->topLeft.y);
  FntSetFont(stdFont);
}

static void FixtureFormInitTables1(void)
{
  Int16 g, n;
  struct match match[6];
  struct teamState standings[4];
  TablePtr fixtureTable = Id2Ptr(IdFrmFixture);

  for (g=0; g<4; g++) {
    TblSetRowUsable(fixtureTable, g, true);
    TblSetRowSelectable(fixtureTable, g, false);
    ReadSetFixture(match,g+1,false);
    ComputeTable(standings, match, 0xffffffff);
    for (n=0; n<4; n++) {
      TblSetItemInt(fixtureTable, n, g, g*4+standings[n].teamNo);
      TblSetItemStyle(fixtureTable, n, g, customTableItem);
      TblSetColumnUsable(fixtureTable, g, true);
      TblSetCustomDrawProcedure(fixtureTable, g, DrawFixtureTableItem);
    }
  }

}

static void FixtureFormInitTables2(void)
{
  Int16 team1, team2;
  struct matchRecord matchRec;
  char *p, *content;
  Int16 currentField = IdFrmFixture+10;
  struct team* team;

  for (p=ReadFixture(0, &matchRec); p; p=ReadFixture(p, &matchRec)) {
    if (matchRec.group != 0)
      continue;

    team1 = ComputeSetHandleString(0, matchRec.m1);
    team2 = ComputeSetHandleString(0, matchRec.m2);

    if (team1 < 64) {
      team = GetTeamString(team1);
      content = team->abbr;
    } else {
      content = GetString(IdFinalsShortName+matchRec.m1);
    }
    if (content)
      FldSetTextPtr(Id2Ptr(currentField),content);
    currentField++;

    if (team2 < 64) {
      team = GetTeamString(team2);
      content = team->abbr;
    } else {
      content = GetString(IdFinalsShortName+matchRec.m2);
    }
    if (content)
      FldSetTextPtr(Id2Ptr(currentField),content);
    currentField++;
  }
}

Boolean FixtureFormHandleEvent(EventPtr e)
{
  Boolean handled;
  RGBColorType rgb, save;
  RectangleType rect;
  
  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    if (FeatureSet35) {
      UIColorGetTableEntryRGB(UIFieldBackground, &save);
      rgb.r = 0x98; rgb.g = 0xcc; rgb.b = 0xf8;
      UIColorSetTableEntry(UIFieldBackground, &rgb);
    } 

    FixtureFormInitTables1();
    FixtureFormInitTables2();
    FrmDrawForm(FrmGetActiveForm());

    DrawButton(IdFrmFixture+2,IdBitmap+10);
    DrawButton(IdFrmFixture+3,IdBitmap+12);
    DrawButton(IdFrmFixture+4,IdBitmap+15);

    if (FeatureSet35) {
      UIColorSetTableEntry(UIFieldBackground, &save);
    } else {
      rect.topLeft.x = 1;  rect.extent.x = 158;
      rect.topLeft.y = 15; rect.extent.y = 67;
      WinDrawRectangleFrame(simpleFrame,&rect);
    }

    handled = true; 
    break;

  case menuEvent:

    switch(e->data.menu.itemID) {

    case IdMenuItem:
      FrmPopupForm(IdFrmAbout);
      break;

    case IdMenuItem+2:
      FrmPopupForm(IdFrmTimezone);
      break;

    case IdMenuItem+3:
      FrmGotoForm(IdFrmMain);
      break;

    case IdMenuItem+4:
      FrmGotoForm(IdFrmMatches);
      break;

    case IdMenuItem+5:
      BeamDatabase();
      break;

    }
    handled = true;
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {

    case IdFrmFixture+2: // "Groups form" button
      FrmGotoForm(IdFrmMain);
      break;

    case IdFrmFixture+3: // "Matches form" button
      FrmGotoForm(IdFrmMatches);
      break;

    default:
      /* (i) button => show popup with venue and date of match */
      if (e->data.ctlEnter.controlID >= IdFrmFixture+30 &&
	  e->data.ctlEnter.controlID <= IdFrmFixture+36) {
	selectedGroup = 0;
	ReadSetFixture(match,selectedGroup,false);
	selectedMatch = e->data.ctlEnter.controlID - IdFrmFixture - 30;
	FrmPopupForm(IdFrmMainPopup);
      }
    }
    handled = true;
    break;

  case keyDownEvent:
    if (e->data.keyDown.chr == vchrRockerLeft ||
	e->data.keyDown.keyCode == fiveWayLeft) {
      FrmGotoForm(IdFrmMatches);
      handled = true;
    }

    if (e->data.keyDown.chr == vchrRockerRight || 
	e->data.keyDown.keyCode == fiveWayRight) {
      FrmGotoForm(IdFrmMain);
      handled = true;
    }

    break;

  default: 
  }

  return handled;
}

