/*
 * Euro 2008 is Copyright (c) 2008 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "euro2008.h"
#include "euro2008rcp.h"


static void ShowStadiumFormInitForm(UInt16 selectedStadium)
{
  void *hBmp;
  Int16 row,fld;
  char *p, buf[10];
  struct matchRecord Match;
  struct team *team;
  struct venue* venue = GetLocation(selectedStadium-1);

  FrmDrawForm(FrmGetActiveForm());

  SetTextFieldWithString(Id2Ptr(IdFrmStadium+1),venue->name,true);
  SetHandleInt(Id2Ptr(IdFrmStadium+2),venue->cap);
  FldDrawField(Id2Ptr(IdFrmStadium+2));
  if ( (hBmp = DmGetResource((UInt32)'Tbmp',IdVenueMap+1)) ) {
    WinDrawBitmap(MemHandleLock(hBmp),venue->x+3,venue->y+68);
    MemHandleUnlock(hBmp);
    DmReleaseResource(hBmp);
  }
  
  // every stadium has between 3 and 7 venues; clear 
  for (fld=6; fld<=9; fld++) {
    SetTextFieldWithString(Id2Ptr(IdFrmStadium+fld)," ",true);
  }

  fld = 3;
  for ( p=ReadFixture(0,&Match),row=0; p; p=ReadFixture(p,&Match),row++ ) {
    if (row>31) continue;
    if (Match.venue == selectedStadium) {
      if (Match.group>0) {
	team = GetTeamString((Match.group-1)*4+Match.m1);
	StrCopy(buf,team->abbr);
	StrCat(buf,"-");
	team = GetTeamString((Match.group-1)*4+Match.m2);
	StrCat(buf,team->abbr);
	SetTextFieldWithString(Id2Ptr(IdFrmStadium+fld),buf,true);
      } else {
	if (Match.m1<25) {
	  StrCopy(buf,"1/4 Fin");
	} else if (Match.m1<29) {
	  StrCopy(buf,"1/2 Fin");
	} else {
	  StrCopy (buf, "Final");
	}
	SetTextFieldWithString(Id2Ptr(IdFrmStadium+fld),buf,true);
      }
      fld++;
    }
  }
}

/**
 * Show Stadium Form
 */
Boolean ShowStadiumFormHandleEvent(EventPtr e)
{
  Boolean handled;
  struct venue* venue;
  Int16 i;
  
  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    ShowStadiumFormInitForm(match[selectedMatch].venue);
    handled = true; 
    break;

  case ctlSelectEvent:
    if (e->data.ctlSelect.controlID == IdFrmStadium) {
      FrmReturnToForm(0);
      handled = true; 
    }
    break;

  case penUpEvent:
    for (i=0;i<8;i++) {
      venue = GetLocation(i);
      if (e->data.penUp.end.x-3 >= venue->x-10 && e->data.penUp.end.x-3 <= venue->x+10 && 
          e->data.penUp.end.y-68 >= venue->y-10 && e->data.penUp.end.y-68 <= venue->y+10) {
	ShowStadiumFormInitForm(i+1);
	handled = true;
	break;
      }
    }
    break;

  default:
    break;
  }

  return handled;
}

